import React, { useState, useRef, useMemo, useImperativeHandle, forwardRef, useEffect } from 'react';
import { VisualizationPropTypes, VISComponentType, VisualizationSettingsPropTypes } from '../../common';
import { SettingsContainer } from '@graphpolaris/shared/lib/vis/components/config';
import html2canvas from 'html2canvas';
import { CustomChartPlotly, plotTypeOptions } from './components/CustomChartPlotly';
import { Input } from '@graphpolaris/shared/lib/components/inputs';
import { DropdownTextAndIcon } from '@graphpolaris/shared/lib/components/selectors/textAndIcon';
import { EntityPill } from '@graphpolaris/shared/lib/components/pills/Pill';
import { Button } from '@graphpolaris/shared/lib/components/buttons';

export interface Vis1DProps {
  plotType: (typeof plotTypeOptions)[number];
  title: string;
  attribute?: string;
  nodeLabel: string;
}

const defaultSettings: Vis1DProps = {
  plotType: 'bar',
  title: '',
  attribute: '',
  nodeLabel: '',
};

export interface Vis1DVisHandle {
  exportImageInternal: () => void;
}
const Vis1D = forwardRef<Vis1DVisHandle, VisualizationPropTypes<Vis1DProps>>(({ data, settings }, refExternal) => {
  const internalRef = useRef<HTMLDivElement>(null);

  useImperativeHandle(refExternal, () => ({
    exportImageInternal() {
      const captureImage = () => {
        const element = internalRef.current;
        if (element) {
          html2canvas(element, {
            backgroundColor: '#FFFFFF',
          })
            .then((canvas) => {
              const finalImage = canvas.toDataURL('image/png');
              const link = document.createElement('a');
              link.href = finalImage;
              link.download = 'Vis1D.png';
              document.body.appendChild(link);
              link.click();
              document.body.removeChild(link);
            })
            .catch((error) => {
              console.error('Error capturing image:', error);
            });
        } else {
          console.error('Container element not found');
        }
      };

      const renderCanvas = () => {
        requestAnimationFrame(() => {
          captureImage();
        });
      };

      renderCanvas();
    },
  }));

  const attributeValues = useMemo(() => {
    if (!settings.nodeLabel || !settings.attribute) {
      return [];
    }

    return data.nodes
      .filter((item) => item.label === settings.nodeLabel && item.attributes && settings.attribute! in item.attributes)
      .map((item) => item.attributes[settings.attribute!] as string | number);
  }, [data, settings.nodeLabel, settings.attribute]);

  return (
    <div className="h-full w-full flex items-center justify-center overflow-hidden" ref={internalRef}>
      <CustomChartPlotly data={attributeValues as string[] | number[]} plotType={settings.plotType} title={settings.title} />
    </div>
  );
});

const Vis1DSettings = ({ settings, graphMetadata, updateSettings }: VisualizationSettingsPropTypes<Vis1DProps>) => {
  const mutablePlotTypes = [...plotTypeOptions];
  const [attributeOptions, setAttributeOptions] = useState<{ name: string; type: string }[]>([]);
  const [selectedOption, setSelectedOption] = useState<{ name: string; type: string } | null>(null);

  const handleChange = (option: { name: string; type: string }) => {
    setSelectedOption(option);
    updateSettings({ attribute: option.name });
  };
  useEffect(() => {
    if (graphMetadata && graphMetadata.nodes && graphMetadata.nodes.labels.length > 0) {
      const nodeLabelTemp = graphMetadata.nodes.labels[0];
      updateSettings({ nodeLabel: nodeLabelTemp });
    }
  }, [graphMetadata]);

  useEffect(() => {
    if (graphMetadata && graphMetadata.nodes && graphMetadata.nodes.labels.length > 0 && settings.nodeLabel != '') {
      const newAttributeOptions = Object.entries(graphMetadata.nodes.types[settings.nodeLabel].attributes).map(([key, value]) => ({
        name: key,
        type: value.dimension,
      }));
      updateSettings({ attribute: newAttributeOptions[0].name });
      // initialize the selected option for creating the dropdown and plots
      setSelectedOption(newAttributeOptions[0]);
      setAttributeOptions(newAttributeOptions);
    }
  }, [graphMetadata, settings.nodeLabel]);

  return (
    <SettingsContainer>
      <div className="p-1">
        <Input
          className="w-full text-justify justify-start mb-2"
          type="dropdown"
          value={settings.nodeLabel}
          options={graphMetadata.nodes.labels}
          onChange={(val) => updateSettings({ nodeLabel: val as string })}
          overrideRender={
            <EntityPill
              title={
                <div className="flex flex-row justify-between items-center cursor-pointer">
                  <span>{settings.nodeLabel || ''}</span>
                  <Button variantType="secondary" variant="ghost" size="2xs" iconComponent="icon-[ic--baseline-arrow-drop-down]" />
                </div>
              }
            />
          }
        />

        <div className="mb-2">
          <Input type="text" label="Title" value={settings.title} onChange={(value) => updateSettings({ title: value as string })} />
        </div>
        <div className="mb-2">
          <Input
            type="dropdown"
            label="Type Chart"
            value={settings.plotType}
            options={mutablePlotTypes}
            onChange={(value: string | number) => {
              updateSettings({ plotType: value as (typeof plotTypeOptions)[number] });
            }}
          />
        </div>
        <div className="mb-2">
          <DropdownTextAndIcon value={selectedOption} onChange={handleChange} options={attributeOptions} />
        </div>
      </div>
    </SettingsContainer>
  );
};

const Vis1DRef = React.createRef<Vis1DVisHandle>();

export const Vis1DComponent: VISComponentType<Vis1DProps> = {
  displayName: '1Dvis',
  description: 'Basic charts',
  component: React.forwardRef((props: VisualizationPropTypes<Vis1DProps>, ref) => <Vis1D {...props} ref={Vis1DRef} />),
  settingsComponent: Vis1DSettings,
  settings: defaultSettings,
  exportImage: () => {
    if (Vis1DRef.current) {
      Vis1DRef.current.exportImageInternal();
    } else {
      console.error('1Dvis reference is not set.');
    }
  },
};

export default Vis1DComponent;
